@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 import slapos.slap
 import time
 import subprocess
@@ -11,6 +13,7 @@ import shutil
 import string
 import hashlib
 import signal
+import multiprocessing
 
 
 
@@ -32,13 +35,91 @@ html_escape_table = {
   ">": "&gt;",
   "<": "&lt;",
 }
- 
+
 def html_escape(text):
   """Produce entities within text."""
   return "".join(html_escape_table.get(c,c) for c in text)
 
+def checkLogin(config, login, pwd):
+  """
+  User authentication method
+
+  Args:
+    config: Slaprunner configuration.
+    login: username of the user.
+    pwd: password associate to username.
+
+  Returns:
+    a list of user informations or False if authentication fail.
+    list=[username, password, email, complete_name]
+  """
+  user = getSession(config)
+  salt = "runner81" #to be changed
+  current_pwd = hashlib.md5( salt + pwd ).hexdigest()
+  if current_pwd == user[1]:
+    return user
+  return False
+
+def getSession(config):
+  """
+  Get the session data of current user.
+  Returns:
+    a list of user informations or False if fail to read data.
+  """
+  user_path = os.path.join(config['runner_workdir'], '.users')
+  user = ""
+  if os.path.exists(user_path):
+    user = open(user_path, 'r').read().split(';')
+  if type(user) == type(""):
+    #Error: try to restore data from backup
+    if os.path.exists(user_path+'.back'):
+      os.rename(user_path+'.back', user_path)
+      user = open(user_path, 'r').read().split(';')
+    else:
+      return False
+  return user
+
+def saveSession(config, session, account):
+  """
+  Save account information for the current user
+
+  Args:
+    config: Slaprunner configuration
+    session: Flask session
+    account: New session data to be save
+
+  Returns:
+    True if all goes well or str (error message) if fail
+  """
+  user = os.path.join(config['runner_workdir'], '.users')
+  backup = False
+  try:
+    if account[1]:
+      salt = "runner81" #to be changed
+      account[1] = hashlib.md5(salt + account[1]).hexdigest()
+    else:
+      account[1] = session['account'][1]
+    #backup previous data
+    open(user+'.back', 'w').write(';'.join(session['account']))
+    backup = True
+    #save new account data
+    open(user, 'w').write((';'.join(account)).encode("utf-8"))
+    session['account'] = account
+    return True
+  except Exception, e:
+    try:
+      if backup:
+        os.remove(user)
+        os.rename(user+'.back', user)
+    except:
+      pass
+    return str(e)
 
 def updateProxy(config):
+  """
+  Configure Slapos Node computer and partitions.
+  Send current Software Release to Slapproxy for compilation and deployment.
+  """
   if not os.path.exists(config['instance_root']):
     os.mkdir(config['instance_root'])
   slap = slapos.slap.slap()
@@ -59,8 +140,7 @@ def updateProxy(config):
  'address': config['ipv4_address'],
  'instance_root': config['instance_root'],
  'netmask': '255.255.255.255',
- 'partition_list': [
-                    ],
+ 'partition_list': [],
  'reference': config['computer_id'],
  'software_root': config['software_root']}
   for i in xrange(0, int(config['partition_amount'])):
@@ -81,19 +161,17 @@ def updateProxy(config):
   #get instance parameter
   param_path = os.path.join(config['runner_workdir'], ".parameter.xml")
   xml_result = readParameters(param_path)
+  partition_parameter_kw = None
   if type(xml_result) != type('') and xml_result.has_key('instance'):
     partition_parameter_kw = xml_result['instance']
-  else:
-    partition_parameter_kw = None
   computer.updateConfiguration(xml_marshaller.dumps(slap_config))
-  sr_request = slap.registerOpenOrder().request(profile, partition_reference=partition_reference,
+  sr_request = slap.registerOpenOrder().request(profile, partition_reference=getSoftwareReleaseName(config),
                 partition_parameter_kw=partition_parameter_kw, software_type=None,
                 filter_kw=None, state=None, shared=False)
-  #open(param_path, 'w').write(xml_marshaller.dumps(sr_request.
-  #                      getInstanceParameterDict()))
   return True
 
 def readPid(file):
+  """Read process pid from file `file`"""
   if os.path.exists(file):
     data = open(file).read().strip()
     try:
@@ -104,10 +182,39 @@ def readPid(file):
 
 
 def writePid(file, pid):
+  """Save process pid into a file `file`"""
   open(file, 'w').write(str(pid))
 
+def updateInstanceParameter(config, software_type=None):
+  """
+  Reconfigure Slapproxy to re-deploy current Software Instance with parameters.
+
+  Args:
+    config: Slaprunner configuration.
+    software_type: reconfigure Software Instance with software type.
+  """
+  slap = slapos.slap.slap()
+  slap.initializeConnection(config['master_url'])
+  #Get current software release profile
+  try:
+    software_folder = open(os.path.join(config['runner_workdir'],
+                                        ".project")).read()
+    profile = realpath(config, os.path.join(software_folder,
+                                          config['software_profile']))
+  except:
+    raise Exception("Software Release profile not found")
+  #get instance parameter
+  param_path = os.path.join(config['runner_workdir'], ".parameter.xml")
+  xml_result = readParameters(param_path)
+  partition_parameter_kw = None
+  if type(xml_result) != type('') and xml_result.has_key('instance'):
+    partition_parameter_kw = xml_result['instance']
+  slap.registerOpenOrder().request(profile, partition_reference=getSoftwareReleaseName(config),
+          partition_parameter_kw=partition_parameter_kw, software_type=software_type,
+          filter_kw=None, state=None, shared=False)
 
 def startProxy(config):
+  """Start Slapproxy server"""
   proxy_pid = os.path.join(config['runner_workdir'], 'proxy.pid')
   pid = readPid(proxy_pid)
   running = False
@@ -126,6 +233,7 @@ def startProxy(config):
 
 
 def stopProxy(config):
+  """Stop Slapproxy server"""
   pid = readPid(os.path.join(config['runner_workdir'], 'proxy.pid'))
   if pid:
     try:
@@ -135,10 +243,15 @@ def stopProxy(config):
 
 
 def removeProxyDb(config):
+  """Remove Slapproxy database, this is use to initialize proxy for example when
+    configuring new Software Release"""
   if os.path.exists(config['database_uri']):
     os.unlink(config['database_uri'])
 
 def isSoftwareRunning(config):
+  """
+    Return True if slapgrid-sr is still running and false if slapgrid if not
+  """
   slapgrid_pid = os.path.join(config['runner_workdir'], 'slapgrid-sr.pid')
   pid = readPid(slapgrid_pid)
   if pid:
@@ -154,6 +267,10 @@ def isSoftwareRunning(config):
 
 
 def runSoftwareWithLock(config):
+  """
+    Use Slapgrid to compile current Software Release and wait until
+    compilation is done
+  """
   slapgrid_pid = os.path.join(config['runner_workdir'], 'slapgrid-sr.pid')
   if not isSoftwareRunning(config):
     if not os.path.exists(config['software_root']):
@@ -164,7 +281,12 @@ def runSoftwareWithLock(config):
     logfile = open(config['software_log'], 'w')
     if not updateProxy(config):
       return False
-    slapgrid = Popen([config['slapgrid_sr'], '-vc', config['configuration_file_path']], stdout=logfile)
+    # Accelerate compilation by setting make -jX
+    environment = os.environ.copy()
+    environment['MAKEFLAGS'] = '-j%r' % multiprocessing.cpu_count()
+    slapgrid = Popen([config['slapgrid_sr'], '-vc',
+        config['configuration_file_path'], '--now'],
+        stdout=logfile, env=environment)
     writePid(slapgrid_pid, slapgrid.pid)
     slapgrid.wait()
     #Saves the current compile software for re-use
@@ -191,6 +313,9 @@ def runSoftwareWithLock(config):
 
 
 def isInstanceRunning(config):
+  """
+    Return True if slapgrid-cp is still running and false if slapgrid if not
+  """
   slapgrid_pid = os.path.join(config['runner_workdir'], 'slapgrid-cp.pid')
   pid = readPid(slapgrid_pid)
   if pid:
@@ -205,6 +330,7 @@ def isInstanceRunning(config):
   return running
 
 def killRunningSlapgrid(config, ptype):
+  """Kill slapgrid process and all running children process"""
   slapgrid_pid = os.path.join(config['runner_workdir'], ptype)
   pid = readPid(slapgrid_pid)
   if pid:
@@ -233,25 +359,43 @@ def pidppid(pid):
   return list(int(p) for p, pp in ppid if int(pp) == pid)
 
 def runInstanceWithLock(config):
+  """
+    Use Slapgrid to deploy current Software Release and wait until
+    deployment is done.
+  """
   slapgrid_pid = os.path.join(config['runner_workdir'], 'slapgrid-cp.pid')
   if not isInstanceRunning(config):
     startProxy(config)
     logfile = open(config['instance_log'], 'w')
     if not updateProxy(config):
       return False
-    slapgrid = Popen([config['slapgrid_cp'], '-vc', config['configuration_file_path']], stdout=logfile)
+    svcStopAll(config) #prevent lost control of process
+    slapgrid = Popen([config['slapgrid_cp'], '-vc',
+        config['configuration_file_path'], '--now'],
+        stdout=logfile)
     writePid(slapgrid_pid, slapgrid.pid)
     slapgrid.wait()
     return True
   return False
 
-def getProfilePath(peojectDir, profile):
-  if not os.path.exists(os.path.join(peojectDir, ".project")):
+def getProfilePath(projectDir, profile):
+  """
+  Return the path of the current Software Release `profile`
+
+  Args:
+    projectDir: Slaprunner workspace location.
+    profile: file to search into the workspace.
+
+  Returns:
+    String, path of current Software Release profile
+  """
+  if not os.path.exists(os.path.join(projectDir, ".project")):
     return False
-  projectFolder = open(os.path.join(peojectDir, ".project")).read()
+  projectFolder = open(os.path.join(projectDir, ".project")).read()
   return os.path.join(projectFolder, profile)
 
 def getSlapStatus(config):
+  """Return all Slapos Partitions with associate informations"""
   slap = slapos.slap.slap()
   slap.initializeConnection(config['master_url'])
   partition_list = []
@@ -259,8 +403,7 @@ def getSlapStatus(config):
   try:
     for partition in computer.getComputerPartitionList():
       # Note: Internal use of API, as there is no reflexion interface in SLAP
-      partition_list.append((partition.getId(), partition._connection_dict.copy(),
-                              partition._parameter_dict.copy()))
+      partition_list.append((partition.getId(), partition._connection_dict.copy()))
   except Exception:
     pass
   if partition_list:
@@ -283,10 +426,12 @@ def runBuildoutAnnotate(config):
   return False
 
 def svcStopAll(config):
+  """Stop all Instance process on this computer"""
   return Popen([config['supervisor'], config['configuration_file_path'],
                 'shutdown']).communicate()[0]
 
 def removeInstanceRoot(config):
+  """Clean instance directory and stop all its running process"""
   if os.path.exists(config['instance_root']):
     svcStopAll(config)
     for root, dirs, files in os.walk(config['instance_root']):
@@ -298,6 +443,7 @@ def removeInstanceRoot(config):
     shutil.rmtree(config['instance_root'])
 
 def getSvcStatus(config):
+  """Return all Softwares Instances process Informations"""
   result = Popen([config['supervisor'], config['configuration_file_path'],
                   'status']).communicate()[0]
   regex = "(^unix:.+\.socket)|(^error:).*$"
@@ -311,15 +457,40 @@ def getSvcStatus(config):
   return supervisord
 
 def getSvcTailProcess(config, process):
+  """Get log for the specifie process
+
+  Args:
+    config: Slaprunner configuration
+    process: process name. this value is pass to supervisord.
+  Returns:
+    a string that contains the log of the process.
+  """
   return Popen([config['supervisor'], config['configuration_file_path'],
                 "tail", process]).communicate()[0]
 
 def svcStartStopProcess(config, process, action):
+  """Send start or stop process command to supervisord
+
+  Args:
+    config: Slaprunner configuration.
+    process: process to start or stop.
+    action: current state which is used to generate the new process state.
+  """
   cmd = {"RESTART":"restart", "STOPPED":"start", "RUNNING":"stop", "EXITED":"start", "STOP":"stop"}
   return Popen([config['supervisor'], config['configuration_file_path'],
                 cmd[action], process]).communicate()[0]
 
 def getFolderContent(config, folder):
+  """
+  Read all file and folder into specified directory
+
+  Args:
+    config: Slaprunner configuration.
+    folder: the directory to read.
+
+  Returns:
+    Html formated string or error message when fail.
+  """
   r=['<ul class="jqueryFileTree" style="display: none;">']
   try:
     folder = str(folder)
@@ -347,6 +518,16 @@ def getFolderContent(config, folder):
   return jsonify(result=''.join(r))
 
 def getFolder(config, folder):
+  """
+  Read list of folder for the specified directory
+
+  Args:
+    config: Slaprunner configuration.
+    folder: the directory to read.
+
+  Returns:
+    Html formated string or error message when fail.
+  """
   r=['<ul class="jqueryFileTree" style="display: none;">']
   try:
     folder = str(folder)
@@ -371,6 +552,13 @@ def getFolder(config, folder):
   return jsonify(result=''.join(r))
 
 def getProjectList(folder):
+  """Return the list of projet (folder) into the workspace
+
+  Agrs:
+    folder: path of the workspace
+  Returns:
+    a list that contains each folder name.
+  """
   project = []
   project_list = sorted(os.listdir(folder), key=str.lower)
   for elt in project_list:
@@ -378,6 +566,14 @@ def getProjectList(folder):
   return project
 
 def configNewSR(config, projectpath):
+  """Configure a Software Release as current Software Release
+
+  Args:
+    config: slaprunner configuration
+    projectpath: path of the directory that contains the software realease to configure
+  Returns:
+    True if all is done well, otherwise return false.
+  """
   folder = realpath(config, projectpath)
   if folder:
     if isInstanceRunning(config):
@@ -388,12 +584,22 @@ def configNewSR(config, projectpath):
     removeProxyDb(config)
     startProxy(config)
     removeInstanceRoot(config)
+    param_path = os.path.join(config['runner_workdir'], ".parameter.xml")
+    if os.path.exists(param_path):
+      os.remove(param_path)
     open(os.path.join(config['runner_workdir'], ".project"), 'w').write(projectpath)
     return True
   else:
     return False
 
 def newSoftware(folder, config, session):
+  """
+  Create a new Software Release folder with default profiles
+
+  Args:
+    folder: directory of the new software release
+    config: slraprunner configuration
+    session: Flask session directory"""
   json = ""
   code = 0
   runner_dir = config['runner_workdir']
@@ -427,12 +633,14 @@ def newSoftware(folder, config, session):
   return jsonify(code=code, result=json)
 
 def checkSoftwareFolder(path, config):
+  """Check id `path` is a valid Software Release folder"""
   realdir = realpath(config, path)
   if realdir and os.path.exists(os.path.join(realdir, config['software_profile'])):
     return jsonify(result=path)
   return jsonify(result="")
 
 def getProjectTitle(config):
+  """Generate the name of the current software Release (for slaprunner UI)"""
   conf = os.path.join(config['runner_workdir'], ".project")
   if os.path.exists(conf):
     project = open(conf, "r").read().split("/")
@@ -440,7 +648,22 @@ def getProjectTitle(config):
     return software + " (" + string.join(project[:(len(project) - 2)], '/') + ")"
   return "No Profile"
 
+def getSoftwareReleaseName(config):
+  """Get the name of the current Software Release"""
+  sr_profile = os.path.join(config['runner_workdir'], ".project")
+  if os.path.exists(sr_profile):
+    project = open(sr_profile, "r").read().split("/")
+    software = project[len(project) - 2]
+    return software.replace(' ', '_')
+  return "No_name"
+
 def loadSoftwareData(runner_dir):
+  """Get All Compiled Softwares Releases name and directory
+
+  Agrs:
+    runner_dir: base directory of slapos web runner.
+  Returns:
+    a dictionnary that contains all compiled Software Release with path"""
   import pickle
   file_path = os.path.join(runner_dir, '.softdata')
   if not os.path.exists(file_path):
@@ -451,6 +674,12 @@ def loadSoftwareData(runner_dir):
   return data
 
 def writeSoftwareData(runner_dir, data):
+  """Save the list of compiled Software Release into a file
+
+  Args:
+    runner_dir: base directory of slapos web runner.
+    data: dictionnary data about real name and directory of each software release
+  """
   import pickle
   file_path = os.path.join(runner_dir, '.softdata')
   pkl_file = open(file_path, 'wb')
@@ -459,11 +688,16 @@ def writeSoftwareData(runner_dir, data):
   pkl_file.close()
 
 def removeSoftwareByName(config, folderName):
+  """Remove all content of the specified software release
+
+  Args:
+    config: slaprunner configuration
+    foldername: the name given to the software release"""
   if isSoftwareRunning(config) or isInstanceRunning(config):
-    return jsonify(code=0, result="Software installation or instantiation in progress, cannot remove")
+    raise Exception("Software installation or instantiation in progress, cannot remove")
   path = os.path.join(config['software_root'], folderName)
   if not os.path.exists(path):
-    return jsonify(code=0, result="Can not remove software: No such file or directory")
+    raise Exception("Cannot remove software Release: No such file or directory")
   svcStopAll(config)
   shutil.rmtree(path)
   #update compiled software list
@@ -475,7 +709,7 @@ def removeSoftwareByName(config, folderName):
       writeSoftwareData(config['runner_workdir'], data)
       break
     i = i+1
-  return jsonify(code=1, result=data)
+  return data
 
 def tail(f, lines=20):
   """
@@ -583,6 +817,13 @@ def realpath(config, path, check_exist=True):
   return False
 
 def readParameters(path):
+  """Read Instance parameters stored into a local file.
+
+  Agrs:
+    path: path of the xml file that contains parameters
+
+  Return:
+    a dictionnary of instance parameters."""
   if os.path.exists(path):
     try:
       xmldoc = minidom.parse(path)
@@ -591,8 +832,7 @@ def readParameters(path):
         sub_object = {}
         for subnode in elt.childNodes:
           if subnode.nodeType != subnode.TEXT_NODE:
-            sub_object[str(subnode.getAttribute('id'))] = str(subnode.
-	                                                 childNodes[0].data)
+            sub_object[str(subnode.getAttribute('id'))] = subnode.childNodes[0].data #.decode('utf-8').decode('utf-8')
 	    object[str(elt.tagName)] = sub_object
       return object
     except Exception, e:
